package org.hsweb.web.controller.user;

import org.hsweb.web.authorize.annotation.Authorize;
import org.hsweb.web.bean.common.QueryParam;
import org.hsweb.web.bean.po.user.User;
import org.hsweb.web.controller.GenericController;
import org.hsweb.web.exception.NotFoundException;
import org.hsweb.web.logger.annotation.AccessLogger;
import org.hsweb.web.message.ResponseMessage;
import org.hsweb.web.service.user.UserService;
import org.springframework.stereotype.Service;
import org.springframework.web.bind.annotation.PathVariable;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;

import javax.annotation.Resource;

/**
 * 后台管理用户控制器，继承自GenericController,使用rest+json
 * Created by generator 2015-8-26 10:35:57
 */
@RestController
@RequestMapping(value = "/user")
@AccessLogger("用户管理")
@Authorize(module = "user")
public class UserController extends GenericController<User, String> {

    //默认服务类
    @Resource
    private UserService userService;

    @Override
    public UserService getService() {
        return this.userService;
    }

    @Override
    @AccessLogger("获取列表")
    public ResponseMessage list(QueryParam param) throws Exception {
        param.excludes("password");
        return super.list(param)
                .exclude(User.class, "password", "modules", "userRoles")
                .onlyData();
    }

    @Override
    @AccessLogger("获取用户详情")
    public ResponseMessage info(@PathVariable("id") String id) throws Exception {
        return super.info(id).exclude(User.class, "password", "modules");
    }

    @Override
    @AccessLogger("禁用")
    public ResponseMessage delete(@PathVariable("id") String id) throws Exception {
        User user = getService().selectByPk(id);
        if (user == null) throw new NotFoundException("用户不存在!");
        user.setStatus(-1);
        getService().update(user);
        return ResponseMessage.ok( "禁用成功");
    }


}
